#!/bin/bash

# URL del archivo de firmwares
firmwares_url="http://gamba.dynns.com:5000/apk/starfive/firmwares.gz"

# Directorio temporal para los firmwares
firmwares_dir="/tmp/firmwares"

# Firmwares disponibles
firmwares_disponibles=(
    "1. VisionFive2 Software v2.4.4   (Dec 08 2022 - 17:27:48 +0800)"
    "2. VisionFive2 Software v2.5.0   (Dec 25 2022 - 20:59:18 +0800)"
    "3. VisionFive2 Software v2.6.0   (Jan 08 2023 - 18:04:54 +0800)"
    "4. VisionFive2 Software v2.8.0   (Jan 19 2023 - 04:09:41 +0800) Recomendado"
    "5. VisionFive2 Software v2.10.4  (Feb 27 2023 - 21:59:07 +0800)"
    "6. VisionFive2 Software v2.11.15 (Mar 24 2023 - 01:42:56 +0800)"
    "7. VisionFive2 Software v3.0.4   (May 31 2023 - 16:54:00 +0800)"
    "8. VisionFive2 Software v3.1.5   (Jun 21 2023 - 13:42:04 +0800)"
)

# Función para mostrar el menú principal en español
mostrar_menu_es() {
    clear
    echo -e "\e[1m\e[36m====== MENÚ PRINCIPAL ======\e[0m"
    echo -e "Seleccione una opción:"
    echo -e "\e[33m[1]\e[0m Mostrar la versión del firmware actual"
    echo -e "\e[33m[2]\e[0m Descargar firmwares"
    echo -e "\e[33m[3]\e[0m Ver firmwares disponibles"
    echo -e "\e[33m[4]\e[0m Actualizar firmware"
    echo -e "\e[33m[0]\e[0m Salir"
    echo -e "\e[1m\e[36m==========================\e[0m"
}

# Función para mostrar el menú principal en inglés
mostrar_menu_en() {
    clear
    echo -e "\e[1m\e[36m====== MAIN MENU ======\e[0m"
    echo -e "Select an option:"
    echo -e "\e[33m[1]\e[0m Show current firmware version"
    echo -e "\e[33m[2]\e[0m Download firmwares"
    echo -e "\e[33m[3]\e[0m View available firmwares"
    echo -e "\e[33m[4]\e[0m Update firmware"
    echo -e "\e[33m[0]\e[0m Exit"
    echo -e "\e[1m\e[36m=======================\e[0m"
}

# Función para mostrar un mensaje de opción inválida en español
opcion_invalida_es() {
    echo -e "\e[1m\e[31mOpción inválida. Inténtelo de nuevo.\e[0m"
    sleep 2
}

# Función para mostrar un mensaje de opción inválida en inglés
opcion_invalida_en() {
    echo -e "\e[1m\e[31mInvalid option. Please try again.\e[0m"
    sleep 2
}

# Función para ejecutar la opción seleccionada
ejecutar_opcion() {
    local opcion=$1
    case $opcion in
        1)
            if [ "$idioma" = "es" ]; then
                echo -e "\e[1m\e[36mHa seleccionado la Opción 1.\e[0m"
                echo "Obteniendo la versión actual del firmware..."
            else
                echo -e "\e[1m\e[36mYou have selected Option 1.\e[0m"
                echo "Getting current firmware version..."
            fi
            sleep 2
            echo -e "Your current version is:"
            if [ "$idioma" = "es" ]; then
                dd if=/dev/mtd0 status=none | strings | grep "U-Boot SPL"
            else
                dd if=/dev/mtd0 status=none | strings | grep "U-Boot SPL"
            fi
            pausa
            ;;
        2)
            if [ "$idioma" = "es" ]; then
                echo -e "\e[1m\e[36mHa seleccionado la Opción 2.\e[0m"
                echo "Descargando firmwares..."
            else
                echo -e "\e[1m\e[36mYou have selected Option 2.\e[0m"
                echo "Downloading firmwares..."
            fi
            sleep 2
            
            # Descargar el archivo de firmwares
            if [ "$idioma" = "es" ]; then
                echo "Descargando el archivo de firmwares..."
            else
                echo "Downloading firmwares..."
            fi
            wget -q "$firmwares_url" -P "$firmwares_dir"
            
            # Descomprimir el archivo de firmwares
            if [ "$idioma" = "es" ]; then
                echo "Descomprimiendo el archivo de firmwares..."
            else
                echo "Extracting firmwares file..."
            fi
            tar -xzf "$firmwares_dir/firmwares.gz" -C "$firmwares_dir"
            
            if [ "$idioma" = "es" ]; then
                echo "La descarga de firmwares ha finalizado."
            else
                echo "Firmware download completed."
            fi
            
            pausa
            ;;
        3)
            if [ "$idioma" = "es" ]; then
                echo -e "\e[1m\e[36mHa seleccionado la Opción 3.\e[0m"
                echo "Listando firmwares disponibles..."
            else
                echo -e "\e[1m\e[36mYou have selected Option 3.\e[0m"
                echo "Listing available firmwares..."
            fi
            sleep 2
            if [ "$idioma" = "es" ]; then
                echo -e "\e[1m\e[36m==== Firmwares Disponibles ====\e[0m"
            else
                echo -e "\e[1m\e[36m==== Available Firmwares ====\e[0m"
            fi
            for firmware in "${firmwares_disponibles[@]}"; do
                echo "$firmware"
            done
            if [ "$idioma" = "es" ]; then
                echo -e "\e[1m\e[36m==============================\e[0m"
            else
                echo -e "\e[1m\e[36m=============================\e[0m"
            fi
            pausa
            ;;
        4)
            if [ "$idioma" = "es" ]; then
                echo -e "\e[1m\e[36mHa seleccionado la Opción 4.\e[0m"
                echo "Iniciando el proceso de actualización del firmware..."
            else
                echo -e "\e[1m\e[36mYou have selected Option 4.\e[0m"
                echo "Starting firmware update process..."
            fi
            echo
            
            if [ "$idioma" = "es" ]; then
                echo -e "\e[1m\e[36m==== Firmwares Disponibles ====\e[0m"
            else
                echo -e "\e[1m\e[36m==== Available Firmwares ====\e[0m"
            fi
            for firmware in "${firmwares_disponibles[@]}"; do
                echo "$firmware"
            done
            if [ "$idioma" = "es" ]; then
                echo -e "\e[1m\e[36m==============================\e[0m"
                read -p "Ingrese el número de firmware que desea actualizar: " numero_firmware
            else
                echo -e "\e[1m\e[36m=============================\e[0m"
                read -p "Enter the firmware number you want to update: " numero_firmware
            fi
            
            # Validar si el número de firmware es válido
            if [[ "$numero_firmware" =~ ^[1-9]$ ]]; then
                # Actualizar el firmware seleccionado
                case $numero_firmware in
                    1)
                        firmware_dir="/tmp/firmwares/VF2_v2.4.4/"
                        ;;
                    2)
                        firmware_dir="/tmp/firmwares/VF2_v2.5.0/"
                        ;;
                    3)
                        firmware_dir="/tmp/firmwares/VF2_v2.6.0/"
                        ;;
                    4)
                        firmware_dir="/tmp/firmwares/VF2_v2.8.0/"
                        ;;
                    5)
                        firmware_dir="/tmp/firmwares/VF2_v2.10.4/"
                        ;;
                    6)
                        firmware_dir="/tmp/firmwares/VF2_v2.11.15/"
                        ;;
                    7)
                        firmware_dir="/tmp/firmwares/VF2_v3.0.4/"
                        ;;
                    8)
                        firmware_dir="/tmp/firmwares/VF2_v3.1.5/"
                        ;;
                    *)
                        if [ "$idioma" = "es" ]; then
                            echo "El número de firmware seleccionado no es válido."
                        else
                            echo "Invalid firmware number selected."
                        fi
                        pausa
                        continue
                        ;;
                esac
                
                # Validar si el directorio de firmware existe
                if [ -d "$firmware_dir" ]; then
                    if [ "$idioma" = "es" ]; then
                        echo "Actualizando el firmware número $numero_firmware..."
                    else
                        echo "Updating firmware number $numero_firmware..."
                    fi
                    
                    # Actualizar el firmware utilizando flashcp
                    flashcp -v "$firmware_dir/u-boot-spl.bin.normal.out" /dev/mtd0
                    flashcp -v "$firmware_dir/visionfive2_fw_payload.img" /dev/mtd1
                    
                    if [ "$idioma" = "es" ]; then
                        echo "La actualización ha sido exitosa."
                    else
                        echo "Update successful."
                    fi
                else
                    if [ "$idioma" = "es" ]; then
                        echo "El número de firmware seleccionado no es válido."
                    else
                        echo "Invalid firmware number selected."
                    fi
                fi
            else
                if [ "$idioma" = "es" ]; then
                    echo "El número de firmware seleccionado no es válido."
                else
                    echo "Invalid firmware number selected."
                fi
            fi
            pausa
            ;;

        0)
            if [ "$idioma" = "es" ]; then
                echo -e "\e[1m\e[36mSaliendo del script.\e[0m"
            else
                echo -e "\e[1m\e[36mExiting the script.\e[0m"
            fi
            exit 0
            ;;
        *)
            if [ "$idioma" = "es" ]; then
                opcion_invalida_es
            else
                opcion_invalida_en
            fi
            ;;
    esac
}

pausa() {
    if [ "$idioma" = "es" ]; then
        echo -e "\e[1m\e[33mPresione una tecla para continuar...\e[0m"
    else
        echo -e "\e[1m\e[33mPress any key to continue...\e[0m"
    fi
    read -n 1 -s -r
}

# Verificación del idioma seleccionado por el usuario
if [ "$1" = "en" ]; then
    idioma="en"
else
    idioma="es"
fi

# Crear el directorio temporal para los firmwares
mkdir -p "$firmwares_dir"

# Bucle principal del script
while true; do
    if [ "$idioma" = "es" ]; then
        mostrar_menu_es
    else
        mostrar_menu_en
    fi
    if [ "$idioma" = "es" ]; then
        read -p "Ingrese el número de opción: " opcion
    else
        read -p "Enter option number: " opcion
    fi
    ejecutar_opcion $opcion
    echo
done
